import java.awt.EventQueue;
import java.awt.Graphics;
import java.awt.Point;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JSpinner;
import javax.swing.JComboBox;
import java.awt.Color;
import javax.swing.border.MatteBorder;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.util.ArrayList;
import java.util.List;
import java.awt.event.ActionEvent;
import javax.swing.DefaultComboBoxModel;
import java.awt.Graphics;
import javax.swing.JScrollPane;
import javax.swing.JRadioButton;
import javax.swing.JCheckBox;

/*
 * Diese Klasse ist fuer die Benutzeroberflaeche des Programms zustaendig. Sie uebergibt alle Parameter an die Klasse SierpinskiTriange
 * welche dann die zu zeichnende Flaeche berechnet und zurueck an diese Klasse gibt. Dann visualisiert diese Klasse das Fraktal.
 */
public class SierpinskiTriangleWindow {

	private JFrame frame; //Hauptfenster
	private JPanel panel; //Bereich zum einstellen des Sierpinski Dreiecks
	private JSpinner sSize; //Auswahl fuer die groesse einer Zelle
	private JSpinner sGenerations; //Auswahl fuer die Anzahl der Generationen
	private JComboBox<String> cbRule; //Auswahl fuer die angewandte Regel
	private Grid pDrawArea; //Bereich in dem das Fraktal visualisiert wird
	private JCheckBox cbGrid;  //Auswahl fuer das dazuschalten des Gitters
	private int cellSize = 5; //Variable fuer die groesse der Zelle
	private Color gridColor = Color.GRAY; //Variable fuer die Farbe des Gitters
	
	//Eine Klasse, welche für das Zeichnen des Gitters ausgefuehrt wird, in welchem dann das Sierpinski Dreieck visualisiert wird.
	public class Grid extends JPanel {

        private List<Point> fillCells; //Liste aller Zellen im Gitter

        public Grid() {
            fillCells = new ArrayList<>(1); //Erstellen einer Leeren Liste
        }

        @Override
        protected void paintComponent(Graphics g) {
            super.paintComponent(g); //Uebergeordneter Aufruf der Zeichenmethode des Panels
            int drawArea = frame.getWidth()-175; //Einstellen der Zeichenflaeche
            for (Point fillCell : fillCells) {  //Schleife um alle Zellen zu traversieren
                int cellX = cellSize + (fillCell.x * cellSize); //x-Koordinate 
                int cellY = cellSize + (fillCell.y * cellSize); //y-Koordinate
                g.setColor(Color.BLACK); //Einstellen der Zellfarbe
                g.fillRect(cellX, cellY, cellSize, cellSize); //Faerben der Zelle
            }
            g.setColor(gridColor); //Einstellen der Gitter Farbe
            g.drawRect(cellSize, cellSize, drawArea, drawArea); //Zeichnen des Bereichs
            
            //Zeichnen der vertikalen Linien
            for (int i = cellSize; i <= drawArea; i += cellSize) {
                g.drawLine(i, cellSize, i, drawArea + cellSize); //Zeichnen einer vertikalen Linie
            }
			//Zeichnen der Horizontalen Linien
            for (int i = cellSize; i <= drawArea; i += cellSize) {
                g.drawLine(cellSize, i, drawArea + cellSize, i); //Zeichnen einer horizontalen Linie
            }
        }

        //Funktion zum hinzufuegen eines gefaerbten Punktes
        public void fillCell(int x, int y) {
            fillCells.add(new Point(x, y)); //Hinzufuegen des Punktes an der Stelle (x,y)
            repaint(); //Neuzeichnen des Fraktals
        }

    }

	/**
	 * Start des Programms
	 */
	public static void main(String[] args) { //main-Funktion
		EventQueue.invokeLater(new Runnable() { 
			public void run() { //Startfunktion
				try { //try-catch-block auf Grund von moeglichen Fehlern beim starten (Von Eclipse hinzugefuegt)
					SierpinskiTriangleWindow window = new SierpinskiTriangleWindow(); //Starten des Hauptfensters
					window.frame.setVisible(true); //Sichtbar machen des Hauptfensters
				} catch (Exception e) {
					e.printStackTrace(); //Ausgabe vom Fehlerbericht (Von Eclipse hinzugefuegt)
				}
			}
		});
	}

	//Erstellen der Benutzeroberflaeche
	public SierpinskiTriangleWindow() { 
		initialize(); //Aufruf des Oberflaechenaufbaus
	}

	//Funktion fuer Oberflaechenaufbau
	private void initialize() {
		frame = new JFrame(); //Initialisieren des Hauptfensters
		frame.addComponentListener(new ComponentListener() { //Hinzufuegen eines ComponentListeners fuer eine dynamische Aenderung der Fenstergroesse
			
			@Override 
			public void componentShown(ComponentEvent e) { //NICHT VON BEDEUTUNG
				// TODO Auto-generated method stub
				
			}
			
			@Override
			public void componentResized(ComponentEvent e) {
				panel.setBounds(frame.getWidth()-175, 0, 175, frame.getHeight()); //Dieser Aufruf sorgt dafuer, dass sich das Feld zum einstellen an die Groesse des Hauptfensters anpasst
				
			}
			
			@Override
			public void componentMoved(ComponentEvent e) { //NICHT VON BEDEUTUNG
				// TODO Auto-generated method stub
				
			}
			
			@Override
			public void componentHidden(ComponentEvent e) { //NICHT VON BEDEUTUNG
				// TODO Auto-generated method stub
				
			}
		});
		frame.getContentPane().setBackground(Color.WHITE); //Aendern der Hintergrundfarbe des Hauptfensters
		frame.setBounds(100, 100, 1400, 747); //Einstellen der Groesse des Hauptfensters
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE); //Beim druecken auf das Schliesssymbol wird das Programm beendet
		frame.getContentPane().setLayout(null); //Einstellen des Layout fuer das Platzieren der nachfolgenden Komponenten
		
		panel = new JPanel(); //Initialisieren des Einstellungsfensters
		panel.setBorder(new MatteBorder(1, 1, 1, 1, (Color) new Color(0, 0, 0))); //Einstellen des Fensterdesigns
		panel.setBackground(Color.LIGHT_GRAY); //Einstellen der Fensterfarbe
		panel.setBounds(frame.getWidth()-175, 0, 175, frame.getHeight()); //Einstellen der Fenstergroesse
		frame.getContentPane().add(panel); //Hinzufuegen des Einstellungsfensters in das Hauptfenster
		panel.setLayout(null); //Einstellen des Layouts fuer das Platzieren der nachfolgenden Komponenten
		
		JButton btnStart = new JButton("Visualize"); //Erstellen des "Visualize"-Buttons
		btnStart.addActionListener(new ActionListener() { //Hinzufuegen eines ActionListeners, welcher auf Interaktion mit dem Button ein Kommando ausfuehrt 
			public void actionPerformed(ActionEvent e) { //Kommando, welches bei Button interaktion ausgefuehrt wird
				if (cbGrid.isSelected()) { //Abfrage, ob die Gitter Einstellung aktiviert ist
					gridColor = Color.GRAY; //Aendern der Gitterfarbe in ein sichtbares Grau
				}
				else { //Ansonsten
					gridColor = pDrawArea.getBackground(); //Aendern der Gitterfarbe zu der selben Farbe wie der Hintergrund, um dieses nicht mehr anzuzeigen
				}
				int rule = Integer.parseInt(cbRule.getItemAt(cbRule.getSelectedIndex())); //Variable, in welches die ausgewaehlte Regel als Zahl gespeichert wird
				cellSize = (int) sSize.getValue(); //Setze die Zellgroessenvariable auf den eingestellten Wert
				int generations = (int) sGenerations.getValue(); //Setze deb Wert der Varuabke fuer die Anzahl der Generationen auf den eingestellten Wert
				SierpinskiTriangle sT = new SierpinskiTriangle(rule, (frame.getWidth()-176)/cellSize, generations); //Aufruf der SierpinskiTriangle Klasse mit den eingestellten Parametern
				sT.drawSierpinski(); //ausfuehren der Berechnungsmethode in der SierpinskiTriangle Klasse 
				visualizeSierpinski(sT.getArea(), (frame.getWidth()-176)/cellSize, generations); //Visualisieren des berechneten Fraktals
				
			}
		});
		btnStart.setBounds(6, 150, 156, 29); //Positionsangabe des "Visualize"-Buttons auf der Konfigurationsflaeche
		panel.add(btnStart); //Hinzufuegen des Buttons zur Konfigurationsflaeche
		
		JLabel lbSize = new JLabel("Cell Size"); //Erstellen einer Beschriftung fuer die Zellgroesse
		lbSize.setBounds(6, 29, 61, 16); //Positionsangaben fuer die Beschriftung
		panel.add(lbSize);  //Hinzufuegen der Komponente zur Konfigurationsflaeche
		
		JLabel lblGenerations = new JLabel("Generations"); //Erstellen einer Beschriftung fuer die Generationen
		lblGenerations.setBounds(6, 91, 87, 16); //Positionsangabe fuer die Beschriftung
		panel.add(lblGenerations); //Hinzufuegen der Komponente zur Konfigurationsflaeche
		
		JLabel lblRule = new JLabel("Rule"); //Erstellen einer Beschriftung fuer die Regel
		lblRule.setBounds(6, 123, 61, 16); //Positionsangaben fuer die Beschriftung
		panel.add(lblRule); //Hinzufuegen der Komponente zur Konfigurationsflaeche
		
		sSize = new JSpinner(); //Erstellen einer Komponente zur Auswahl der Zellgroesse
		sSize.setBounds(91, 24, 71, 26); //Positionsangaben fuer die Beschriftung
		panel.add(sSize); //Hinzufuegen der Komponente zur Konfigurationsflaeche
		
		sGenerations = new JSpinner(); //Erstellen einer Komponente zur Auswahl der Generationen
		sGenerations.setBounds(91, 86, 71, 26); //Positionsangaben fuer die Beschriftung
		panel.add(sGenerations); //Hinzufuegen der Komponente zur Konfigurationsflaeche
		
		cbRule = new JComboBox<String>(); //Erstellen einer Komponente zur Auswahl der Regel
		cbRule.setModel(new DefaultComboBoxModel(new String[] {"90", "120", "60", "102", "150", "170", "204", "240"})); //Hinzufuegen der Regeln zur Auswahlkomponente
		cbRule.setBounds(91, 119, 71, 27); //Positionsangaben fuer die Auswahlkomponente
		panel.add(cbRule); //Hinzufuegen der Komponente zur Konfigurationsflaeche
		
		cbGrid = new JCheckBox("Enable grid"); //Erstellen einer Komponente zum einschalten des Gitters
		cbGrid.setBounds(6, 56, 128, 23); //Positionsangaben fuer das Gitter
		panel.add(cbGrid); //Hinzufuegen der Komponente zur Konfigurationsflaeche
		
		pDrawArea = new Grid(); //Erstellen eines Gitters (Aufruf der oberen Klasse)
		pDrawArea.setBounds(0, 0, frame.getWidth()-176, frame.getHeight()); //Positionsangabe fuer das Gitter
		frame.getContentPane().add(pDrawArea); //Hinzufuegen des Gitters an das Hauptfenster
	}

	//Funktion zur Visualisierung des berechneten SierpinskiDreiecks
	protected void visualizeSierpinski(Boolean[][] area, int x, int y) {
		pDrawArea = new Grid(); //Neu Initialisieren des Zeichengitters (Zeichengitter ausleeren)
		pDrawArea.setBounds(0, 0, frame.getWidth()-176, frame.getHeight()); ////Positionsangabe fuer das Gitter, da die alte Position durch das neue initialisieren verloren geht
		frame.getContentPane().add(pDrawArea); //Hinzufuegen des Gitters an das Hauptfenster
		for (int iX = 0; iX < x; iX++) { //Schleife fuer die x-Koordinaten
			for (int iY = 0; iY < y; iY++) { //Schleife fuer die y-Koordinate
				if (area[iX][iY] == Boolean.TRUE) { //Wenn in der berechneten Matrix an der Stelle (x,y) ein "gezeichnete" Zelle Existier, dann
					pDrawArea.fillCell(iX, iY); //Faerbe diese Zelle im Gitter (siehe Grid Klasse)
				}
			}
		}
		
	}
}

